use super::object::Object;
use super::PlainText;
use super::StandardProperties;

#[derive(Debug)]
pub struct Paragraph<'s> {
    pub source: &'s str,
    pub children: Vec<Object<'s>>,
}

#[derive(Debug)]
pub struct Comment<'s> {
    pub source: &'s str,
}

#[derive(Debug)]
pub struct TableCell<'s> {
    pub source: &'s str,
    pub children: Vec<Object<'s>>,
}

#[derive(Debug)]
pub struct VerseBlock<'s> {
    pub source: &'s str,
    pub name: &'s str,
    pub data: Option<&'s str>,
    pub children: Vec<Object<'s>>,
}

#[derive(Debug)]
pub struct CommentBlock<'s> {
    pub source: &'s str,
    pub name: &'s str,
    pub data: Option<&'s str>,
    pub contents: &'s str,
}

#[derive(Debug)]
pub struct ExampleBlock<'s> {
    pub source: &'s str,
    pub name: &'s str,
    pub data: Option<&'s str>,
    pub contents: &'s str,
}

#[derive(Debug)]
pub struct ExportBlock<'s> {
    pub source: &'s str,
    pub name: &'s str,
    pub data: Option<&'s str>,
    pub contents: &'s str,
}

#[derive(Debug)]
pub struct SrcBlock<'s> {
    pub source: &'s str,
    pub name: &'s str,
    pub data: Option<&'s str>,
    pub contents: &'s str,
}

#[derive(Debug)]
pub struct Clock<'s> {
    pub source: &'s str,
}

#[derive(Debug)]
pub struct DiarySexp<'s> {
    pub source: &'s str,
}

#[derive(Debug)]
pub struct Planning<'s> {
    pub source: &'s str,
}

#[derive(Debug)]
pub struct FixedWidthArea<'s> {
    pub source: &'s str,
}

#[derive(Debug)]
pub struct HorizontalRule<'s> {
    pub source: &'s str,
}

#[derive(Debug)]
pub struct Keyword<'s> {
    pub source: &'s str,
    pub key: &'s str,
    pub value: &'s str,
}

#[derive(Debug)]
pub struct BabelCall<'s> {
    pub source: &'s str,
    pub key: &'s str,
    pub value: &'s str,
}

#[derive(Debug)]
pub struct LatexEnvironment<'s> {
    pub source: &'s str,
}

impl<'s> Paragraph<'s> {
    pub(crate) fn of_text(input: &'s str) -> Self {
        let mut objects = Vec::with_capacity(1);
        objects.push(Object::PlainText(PlainText { source: input }));
        Paragraph {
            source: input,
            children: objects,
        }
    }
}

impl<'s> StandardProperties<'s> for Paragraph<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for TableCell<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for Comment<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for VerseBlock<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}
impl<'s> StandardProperties<'s> for CommentBlock<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}
impl<'s> StandardProperties<'s> for ExampleBlock<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}
impl<'s> StandardProperties<'s> for ExportBlock<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}
impl<'s> StandardProperties<'s> for SrcBlock<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for Clock<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for DiarySexp<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for Planning<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for FixedWidthArea<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for HorizontalRule<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for Keyword<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for BabelCall<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for LatexEnvironment<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}
