use super::GetStandardProperties;
use super::StandardProperties;

#[derive(Debug, PartialEq)]
pub enum Object<'s> {
    Bold(Bold<'s>),
    Italic(Italic<'s>),
    Underline(Underline<'s>),
    StrikeThrough(StrikeThrough<'s>),
    Code(Code<'s>),
    Verbatim(Verbatim<'s>),
    PlainText(PlainText<'s>),
    RegularLink(RegularLink<'s>),
    RadioLink(RadioLink<'s>),
    RadioTarget(RadioTarget<'s>),
    PlainLink(PlainLink<'s>),
    AngleLink(AngleLink<'s>),
    OrgMacro(OrgMacro<'s>),
    Entity(Entity<'s>),
    LatexFragment(LatexFragment<'s>),
    ExportSnippet(ExportSnippet<'s>),
    FootnoteReference(FootnoteReference<'s>),
    Citation(Citation<'s>),
    CitationReference(CitationReference<'s>),
    InlineBabelCall(InlineBabelCall<'s>),
    InlineSourceBlock(InlineSourceBlock<'s>),
    LineBreak(LineBreak<'s>),
    Target(Target<'s>),
    StatisticsCookie(StatisticsCookie<'s>),
    Subscript(Subscript<'s>),
    Superscript(Superscript<'s>),
    Timestamp(Timestamp<'s>),
}

#[derive(Debug, PartialEq)]
pub struct Bold<'s> {
    pub source: &'s str,
    pub children: Vec<Object<'s>>,
}

#[derive(Debug, PartialEq)]
pub struct Italic<'s> {
    pub source: &'s str,
    pub children: Vec<Object<'s>>,
}

#[derive(Debug, PartialEq)]
pub struct Underline<'s> {
    pub source: &'s str,
    pub children: Vec<Object<'s>>,
}

#[derive(Debug, PartialEq)]
pub struct StrikeThrough<'s> {
    pub source: &'s str,
    pub children: Vec<Object<'s>>,
}

#[derive(Debug, PartialEq)]
pub struct Code<'s> {
    pub source: &'s str,
    pub contents: &'s str,
}

#[derive(Debug, PartialEq)]
pub struct Verbatim<'s> {
    pub source: &'s str,
    pub contents: &'s str,
}

#[derive(Debug, PartialEq)]
pub struct PlainText<'s> {
    pub source: &'s str,
}

#[derive(Debug, PartialEq)]
pub struct RegularLink<'s> {
    pub source: &'s str,
}

#[derive(Debug, PartialEq)]
pub struct RadioTarget<'s> {
    pub source: &'s str,
    pub children: Vec<Object<'s>>,
}

#[derive(Debug, PartialEq)]
pub struct RadioLink<'s> {
    pub source: &'s str,
    pub children: Vec<Object<'s>>,
}

#[derive(Debug, PartialEq)]
pub struct PlainLink<'s> {
    pub source: &'s str,
    pub link_type: &'s str,
    pub path: &'s str,
}

#[derive(Debug, PartialEq)]
pub struct AngleLink<'s> {
    pub source: &'s str,
    pub link_type: &'s str,
    pub path: &'s str,
}

#[derive(Debug, PartialEq)]
pub struct OrgMacro<'s> {
    pub source: &'s str,
    pub macro_name: &'s str,
    pub macro_args: Vec<&'s str>,
}

#[derive(Debug, PartialEq)]
pub struct Entity<'s> {
    pub source: &'s str,
    pub entity_name: &'s str,
}

#[derive(Debug, PartialEq)]
pub struct LatexFragment<'s> {
    pub source: &'s str,
}

#[derive(Debug, PartialEq)]
pub struct ExportSnippet<'s> {
    pub source: &'s str,
    pub backend: &'s str,
    pub contents: Option<&'s str>,
}

#[derive(Debug, PartialEq)]
pub struct FootnoteReference<'s> {
    pub source: &'s str,
    pub label: Option<&'s str>,
    pub definition: Vec<Object<'s>>,
}

#[derive(Debug, PartialEq)]
pub struct Citation<'s> {
    pub source: &'s str,
}

#[derive(Debug, PartialEq)]
pub struct CitationReference<'s> {
    pub source: &'s str,
}

#[derive(Debug, PartialEq)]
pub struct InlineBabelCall<'s> {
    pub source: &'s str,
}

#[derive(Debug, PartialEq)]
pub struct InlineSourceBlock<'s> {
    pub source: &'s str,
}

#[derive(Debug, PartialEq)]
pub struct LineBreak<'s> {
    pub source: &'s str,
}

#[derive(Debug, PartialEq)]
pub struct Target<'s> {
    pub source: &'s str,
}

#[derive(Debug, PartialEq)]
pub struct StatisticsCookie<'s> {
    pub source: &'s str,
}

#[derive(Debug, PartialEq)]
pub struct Subscript<'s> {
    pub source: &'s str,
}

#[derive(Debug, PartialEq)]
pub struct Superscript<'s> {
    pub source: &'s str,
}

#[derive(Debug, PartialEq)]
pub struct Timestamp<'s> {
    pub source: &'s str,
}

impl<'s> GetStandardProperties<'s> for Object<'s> {
    fn get_standard_properties(&'s self) -> &'s dyn StandardProperties {
        match self {
            Object::Bold(inner) => inner,
            Object::Italic(inner) => inner,
            Object::Underline(inner) => inner,
            Object::StrikeThrough(inner) => inner,
            Object::Code(inner) => inner,
            Object::Verbatim(inner) => inner,
            Object::PlainText(inner) => inner,
            Object::RegularLink(inner) => inner,
            Object::RadioLink(inner) => inner,
            Object::RadioTarget(inner) => inner,
            Object::PlainLink(inner) => inner,
            Object::AngleLink(inner) => inner,
            Object::OrgMacro(inner) => inner,
            Object::Entity(inner) => inner,
            Object::LatexFragment(inner) => inner,
            Object::ExportSnippet(inner) => inner,
            Object::FootnoteReference(inner) => inner,
            Object::Citation(inner) => inner,
            Object::CitationReference(inner) => inner,
            Object::InlineBabelCall(inner) => inner,
            Object::InlineSourceBlock(inner) => inner,
            Object::LineBreak(inner) => inner,
            Object::Target(inner) => inner,
            Object::StatisticsCookie(inner) => inner,
            Object::Subscript(inner) => inner,
            Object::Superscript(inner) => inner,
            Object::Timestamp(inner) => inner,
        }
    }
}

impl<'s> StandardProperties<'s> for Bold<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for Italic<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for Underline<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for StrikeThrough<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for Code<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for Verbatim<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for RegularLink<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for RadioLink<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for RadioTarget<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for PlainLink<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for AngleLink<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for OrgMacro<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for Entity<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for LatexFragment<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for ExportSnippet<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for FootnoteReference<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for Citation<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for CitationReference<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for InlineBabelCall<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for InlineSourceBlock<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for LineBreak<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for Target<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for StatisticsCookie<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for Subscript<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for Superscript<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for Timestamp<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for PlainText<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}
